@@ -2,15 +2,14 @@
  *
  * LOGGING.C - Log file functions for use with Nagios
  *
- * Copyright (c) 1999-2001 Ethan Galstad (nagios@nagios.org)
- * Last Modified:   06-12-2001
+ * Copyright (c) 1999-2005 Ethan Galstad (nagios@nagios.org)
+ * Last Modified:   12-12-2005
  *
  * License:
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,23 +22,25 @@
  *
  *****************************************************************************/
 
-#include "../common/config.h"
-#include "../common/common.h"
-#include "../common/statusdata.h"
-#include "nagios.h"
+#include "../include/config.h"
+#include "../include/common.h"
+#include "../include/statusdata.h"
+#include "../include/nagios.h"
+#include "../include/broker.h"
 
-extern char	log_file[MAX_FILENAME_LENGTH];
-extern char     temp_file[MAX_FILENAME_LENGTH];
-extern char	log_archive_path[MAX_FILENAME_LENGTH];
 
-extern char     *macro_current_service_attempt;
-extern char	*macro_service_state;
-extern char     *macro_current_host_attempt;
-extern char     *macro_host_state;
-extern char     *macro_state_type;
+extern char	*log_file;
+extern char     *temp_file;
+extern char	*log_archive_path;
+
+extern char     *macro_x[MACRO_X_COUNT];
+
+extern host     *host_list;
+extern service  *service_list;
 
 extern int	use_syslog;
 extern int      log_service_retries;
+extern int      log_initial_states;
 
 extern unsigned long      logging_options;
 extern unsigned long      syslog_options;
@@ -61,24 +62,28 @@ extern int      daemon_mode;
 
 /* write something to the log file, syslog, and possibly the console */
 int write_to_logs_and_console(char *buffer, unsigned long data_type, int display){
+	int len;
+	int x;
 
 #ifdef DEBUG0
 	printf("write_to_logs_and_console() start\n");
 #endif
 
-	/* write messages to the log unless we are just verifying the data or testing scheduling... */
-	if(verify_config==FALSE && test_scheduling==FALSE){
-
-		/* write to syslog */
-		write_to_syslog(buffer,data_type);
-
-		/* write to main log */
-		write_to_log(buffer,data_type);
+	/* strip unnecessary newlines */
+	len=strlen(buffer);
+	for(x=len-1;x>=0;x--){
+		if(buffer[x]=='\n')
+			buffer[x]='\x0';
+		else
+			break;
 	        }
 
-	/* should we print to the console? */
-	if(display==TRUE && daemon_mode==FALSE)
-		printf("%s\n",buffer);
+	/* write messages to the logs */
+	write_to_all_logs(buffer,data_type);
+
+	/* write message to the console */
+	if(display==TRUE)
+		write_to_console(buffer);
 
 #ifdef DEBUG0
 	printf("write_to_logs_and_console() end\n");
@@ -88,38 +93,111 @@ int write_to_logs_and_console(char *buffer, unsigned long data_type, int display
         }
 
 
+/* write something to the console */
+int write_to_console(char *buffer){
+
+#ifdef DEBUG0
+	printf("write_to_console() start\n");
+#endif
+
+	/* should we print to the console? */
+	if(daemon_mode==FALSE)
+		printf("%s\n",buffer);
+
+#ifdef DEBUG0
+	printf("write_to_console() end\n");
+#endif
+
+	return OK;
+        }
+
+
+/* write something to the log file and syslog facility */
+int write_to_all_logs(char *buffer, unsigned long data_type){
+
+#ifdef DEBUG0
+	printf("write_to_all_logs() start\n");
+#endif
+
+	/* write to syslog */
+	write_to_syslog(buffer,data_type);
+
+	/* write to main log */
+	write_to_log(buffer,data_type,NULL);
+
+#ifdef DEBUG0
+	printf("write_to_all_logs() end\n");
+#endif
+
+	return OK;
+        }
+
+
+/* write something to the log file and syslog facility */
+int write_to_all_logs_with_timestamp(char *buffer, unsigned long data_type, time_t *timestamp){
+
+#ifdef DEBUG0
+	printf("write_to_all_logs_with_timestamp() start\n");
+#endif
+
+	/* write to syslog */
+	write_to_syslog(buffer,data_type);
+
+	/* write to main log */
+	write_to_log(buffer,data_type,timestamp);
+
+#ifdef DEBUG0
+	printf("write_to_all_logs_with_timestamp() end\n");
+#endif
+
+	return OK;
+        }
+
 
 /* write something to the nagios log file */
-int write_to_log(char *buffer, unsigned long data_type){
+int write_to_log(char *buffer, unsigned long data_type, time_t *timestamp){
 	FILE *fp;
-	time_t t;
+	time_t log_time;
 
 
 #ifdef DEBUG0
 	printf("write_to_log() start\n");
 #endif
 
+	/* don't log anything if we're not actually running... */
+	if(verify_config==TRUE || test_scheduling==TRUE)
+		return OK;
+
 	/* make sure we can log this type of entry */
 	if(!(data_type & logging_options))
 		return OK;
 
 	fp=fopen(log_file,"a+");
 	if(fp==NULL){
-		printf("Warning: Cannot open log file '%s' for writing\n",log_file);
+		if(daemon_mode==FALSE)
+			printf("Warning: Cannot open log file '%s' for writing\n",log_file);
 		return ERROR;
 		}
 
-	/* get the current time */
-	time(&t);
+	/* what timestamp should we use? */
+	if(timestamp==NULL)
+		time(&log_time);
+	else
+		log_time=*timestamp;
 
 	/* strip any newlines from the end of the buffer */
 	strip(buffer);
 
 	/* write the buffer to the log file */
-	fprintf(fp,"[%lu] %s\n",t,buffer);
+	fprintf(fp,"[%lu] %s\n",log_time,buffer);
 
 	fclose(fp);
 
+#ifdef USE_EVENT_BROKER
+	/* send data to the event broker */
+	broker_log_data(NEBTYPE_LOG_DATA,NEBFLAG_NONE,NEBATTR_NONE,buffer,data_type,log_time,NULL);
+#endif
+
 #ifdef DEBUG0
 	printf("write_to_log() end\n");
 #endif
@@ -135,6 +213,10 @@ int write_to_syslog(char *buffer, unsigned long data_type){
 	printf("write_to_syslog() start\n");
 #endif
 
+	/* don't log anything if we're not actually running... */
+	if(verify_config==TRUE || test_scheduling==TRUE)
+		return OK;
+
 	/* bail out if we shouldn't write to syslog */
 	if(use_syslog==FALSE)
 		return OK;
@@ -155,7 +237,7 @@ int write_to_syslog(char *buffer, unsigned long data_type){
 
 
 /* write a service problem/recovery to the nagios log file */
-int log_service_event(service *svc,int state_type){
+int log_service_event(service *svc){
 	char temp_buffer[MAX_INPUT_BUFFER];
 	unsigned long log_options;
 	host *temp_host;
@@ -165,7 +247,7 @@ int log_service_event(service *svc,int state_type){
 #endif
 
 	/* don't log soft errors if the user doesn't want to */
-	if(state_type==SOFT_STATE && !log_service_retries)
+	if(svc->state_type==SOFT_STATE && !log_service_retries)
 		return OK;
 
 	/* get the log options */
@@ -179,16 +261,17 @@ int log_service_event(service *svc,int state_type){
 		log_options=NSLOG_SERVICE_OK;
 
 	/* find the associated host */
-	temp_host=find_host(svc->host_name,NULL);
+	temp_host=find_host(svc->host_name);
 
 	/* grab service macros */
 	clear_volatile_macros();
 	grab_host_macros(temp_host);
 	grab_service_macros(svc);
+	grab_summary_macros(NULL);
 
-	snprintf(temp_buffer,sizeof(temp_buffer),"SERVICE ALERT: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_service_state,(state_type==SOFT_STATE)?"SOFT":"HARD",macro_current_service_attempt,svc->plugin_output);
+	snprintf(temp_buffer,sizeof(temp_buffer),"SERVICE ALERT: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_x[MACRO_SERVICESTATE],macro_x[MACRO_SERVICESTATETYPE],macro_x[MACRO_SERVICEATTEMPT],svc->plugin_output);
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
-	write_to_logs_and_console(temp_buffer,log_options,FALSE);
+	write_to_all_logs(temp_buffer,log_options);
 
 
 #ifdef DEBUG0
@@ -200,7 +283,7 @@ int log_service_event(service *svc,int state_type){
 
 
 /* write a host problem/recovery to the log file */
-int log_host_event(host *hst, int state, int state_type){
+int log_host_event(host *hst){
 	char temp_buffer[MAX_INPUT_BUFFER];
 	unsigned long log_options=0L;
 
@@ -211,39 +294,20 @@ int log_host_event(host *hst, int state, int state_type){
 	/* grab the host macros */
 	clear_volatile_macros();
 	grab_host_macros(hst);
-
-	/* get the host state type macro */
-	if(macro_state_type!=NULL)
-		free(macro_state_type);
-	macro_state_type=(char *)malloc(MAX_STATETYPE_LENGTH);
-	if(macro_state_type!=NULL)
-		strcpy(macro_state_type,(state_type==HARD_STATE)?"HARD":"SOFT");
-
-	/* make sure the host state macro is correct */
-	if(macro_host_state!=NULL)
-		free(macro_host_state);
-	macro_host_state=(char *)malloc(MAX_STATE_LENGTH);
-	if(macro_host_state!=NULL){
-		if(state==HOST_DOWN)
-			strcpy(macro_host_state,"DOWN");
-		else if(state==HOST_UNREACHABLE)
-			strcpy(macro_host_state,"UNREACHABLE");
-		else
-			strcpy(macro_host_state,"UP");
-	        }
+	grab_summary_macros(NULL);
 
 	/* get the log options */
-	if(state==HOST_DOWN)
+	if(hst->current_state==HOST_DOWN)
 		log_options=NSLOG_HOST_DOWN;
-	else if(state==HOST_UNREACHABLE)
+	else if(hst->current_state==HOST_UNREACHABLE)
 		log_options=NSLOG_HOST_UNREACHABLE;
 	else
 		log_options=NSLOG_HOST_UP;
 
 
-	snprintf(temp_buffer,sizeof(temp_buffer),"HOST ALERT: %s;%s;%s;%s;%s\n",hst->name,macro_host_state,macro_state_type,macro_current_host_attempt,hst->plugin_output);
+	snprintf(temp_buffer,sizeof(temp_buffer),"HOST ALERT: %s;%s;%s;%s;%s\n",hst->name,macro_x[MACRO_HOSTSTATE],macro_x[MACRO_HOSTSTATETYPE],macro_x[MACRO_HOSTATTEMPT],hst->plugin_output);
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
-	write_to_logs_and_console(temp_buffer,log_options,FALSE);
+	write_to_all_logs(temp_buffer,log_options);
 
 #ifdef DEBUG0
 	printf("log_host_event() start\n");
@@ -252,6 +316,65 @@ int log_host_event(host *hst, int state, int state_type){
         }
 
 
+/* logs host states */
+int log_host_states(int type, time_t *timestamp){
+	char temp_buffer[MAX_INPUT_BUFFER];
+	host *temp_host;
+
+	/* bail if we shouldn't be logging initial states */
+	if(type==INITIAL_STATES && log_initial_states==FALSE)
+		return OK;
+
+	/* grab summary macros */
+	grab_summary_macros(NULL);
+
+	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
+
+		/* grab the host macros */
+		clear_volatile_macros();
+		grab_host_macros(temp_host);
+
+		snprintf(temp_buffer,sizeof(temp_buffer),"%s HOST STATE: %s;%s;%s;%s;%s\n",(type==INITIAL_STATES)?"INITIAL":"CURRENT",temp_host->name,macro_x[MACRO_HOSTSTATE],macro_x[MACRO_HOSTSTATETYPE],macro_x[MACRO_HOSTATTEMPT],temp_host->plugin_output);
+		temp_buffer[sizeof(temp_buffer)-1]='\x0';
+		write_to_all_logs_with_timestamp(temp_buffer,NSLOG_INFO_MESSAGE,timestamp);
+	        }
+
+	return OK;
+        }
+
+
+/* logs service states */
+int log_service_states(int type, time_t *timestamp){
+	char temp_buffer[MAX_INPUT_BUFFER];
+	service *temp_service;
+	host *temp_host;
+
+	/* bail if we shouldn't be logging initial states */
+	if(type==INITIAL_STATES && log_initial_states==FALSE)
+		return OK;
+
+	/* grab summary macros */
+	grab_summary_macros(NULL);
+
+	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
+
+		/* find the associated host */
+		temp_host=find_host(temp_service->host_name);
+
+		/* grab service macros */
+		clear_volatile_macros();
+		grab_host_macros(temp_host);
+		grab_service_macros(temp_service);
+
+		snprintf(temp_buffer,sizeof(temp_buffer),"%s SERVICE STATE: %s;%s;%s;%s;%s;%s\n",(type==INITIAL_STATES)?"INITIAL":"CURRENT",temp_service->host_name,temp_service->description,macro_x[MACRO_SERVICESTATE],macro_x[MACRO_SERVICESTATETYPE],macro_x[MACRO_SERVICEATTEMPT],temp_service->plugin_output);
+
+		temp_buffer[sizeof(temp_buffer)-1]='\x0';
+		write_to_all_logs_with_timestamp(temp_buffer,NSLOG_INFO_MESSAGE,timestamp);
+	        }
+
+	return OK;
+        }
+
 
 /* rotates the main log file */
 int rotate_log_file(time_t rotation_time){
@@ -305,11 +428,25 @@ int rotate_log_file(time_t rotation_time){
 		return ERROR;
 	        }
 
+#ifdef USE_EVENT_BROKER
+	/* REMOVED - log rotation events are already handled by NEBTYPE_TIMEDEVENT_EXECUTE... */
+	/* send data to the event broker */
+        /*
+	broker_log_data(NEBTYPE_LOG_ROTATION,NEBFLAG_NONE,NEBATTR_NONE,log_archive,log_rotation_method,0,NULL);
+	*/
+#endif
+
 	/* record the log rotation after it has been done... */
 	snprintf(temp_buffer,sizeof(temp_buffer),"LOG ROTATION: %s\n",method_string);
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
-	write_to_logs_and_console(temp_buffer,NSLOG_PROCESS_INFO,FALSE);
+	write_to_all_logs_with_timestamp(temp_buffer,NSLOG_PROCESS_INFO,&rotation_time);
 
+	/* record log file version format */
+	write_log_file_info(&rotation_time);
+
+	/* log current host and service state */
+	log_host_states(CURRENT_STATES,&rotation_time);
+	log_service_states(CURRENT_STATES,&rotation_time);
 
 #ifdef DEBUG3
 	printf("\tRotated main log file to '%s'\n",log_archive);
@@ -321,3 +458,24 @@ int rotate_log_file(time_t rotation_time){
 
 	return OK;
         }
+
+
+/* record log file version/info */
+int write_log_file_info(time_t *timestamp){
+	char temp_buffer[MAX_INPUT_BUFFER];
+
+#ifdef DEBUG0
+	printf("write_log_file_info() start\n");
+#endif
+
+	/* write log version */
+	snprintf(temp_buffer,sizeof(temp_buffer),"LOG VERSION: %s\n",LOG_VERSION_2);
+	temp_buffer[sizeof(temp_buffer)-1]='\x0';
+	write_to_all_logs_with_timestamp(temp_buffer,NSLOG_PROCESS_INFO,timestamp);
+
+#ifdef DEBUG0
+	printf("write_log_file_info() end\n");
+#endif
+
+	return OK;
+        }
