@@ -2,8 +2,8 @@
  *
  * NOTIFICATIONS.C - Service and host notification functions for Nagios
  *
- * Copyright (c) 1999-2004 Ethan Galstad (nagios@nagios.org)
- * Last Modified:   12-05-2004
+ * Copyright (c) 1999-2006 Ethan Galstad (nagios@nagios.org)
+ * Last Modified:   04-07-2006
  *
  * License:
  *
@@ -258,7 +258,7 @@ int check_service_notification_viability(service *svc, int type){
 	host *temp_host;
 	time_t current_time;
 	time_t timeperiod_start;
-	
+
 #ifdef DEBUG0
 	printf("check_service_notification_viability() start\n");
 #endif
@@ -274,6 +274,17 @@ int check_service_notification_viability(service *svc, int type){
 		return ERROR;
 	        }
 
+	/* find the host this service is associated with */
+	temp_host=find_host(svc->host_name);
+
+	/* if we couldn't find the host, return an error */
+	if(temp_host==NULL){
+#ifdef DEBUG4
+		printf("\tCouldn't find the host associated with this service, so we won't send a notification!\n");
+#endif
+		return ERROR;
+	        }
+
 	/* see if the service can have notifications sent out at this time */
 	if(check_time_against_period(current_time,svc->notification_period)==ERROR){
 #ifdef DEBUG4
@@ -341,6 +352,14 @@ int check_service_notification_viability(service *svc, int type){
 			return ERROR;
 	                }
 
+		/* don't send notifications during scheduled downtime */
+		if(svc->scheduled_downtime_depth>0 || temp_host->scheduled_downtime_depth>0){
+#ifdef DEBUG4
+			printf("\tWe shouldn't notify about FLAPPING events during scheduled downtime!\n");
+#endif
+			return ERROR;
+		        }
+
 		/* flapping viability test passed, so the notification can be sent out */
 		return OK;
 	        }
@@ -350,9 +369,6 @@ int check_service_notification_viability(service *svc, int type){
 	/*** NORMAL NOTIFICATIONS ***************/
 	/****************************************/
 
-	/* find the host this service is associated with */
-	temp_host=find_host(svc->host_name);
-
 	/* has this problem already been acknowledged? */
 	if(svc->problem_has_been_acknowledged==TRUE){
 #ifdef DEBUG4
@@ -361,16 +377,6 @@ int check_service_notification_viability(service *svc, int type){
 		return ERROR;
 	        }
 
-#ifdef REMOVED_041403
-	/* do not send a recovery notification if we shouldn't (this flag was set by the check logic) */
-	if(svc->current_state==STATE_OK && svc->no_recovery_notification==TRUE){
-#ifdef DEBUG4
-		printf("\tRecovery notifications should not be sent out for this service!\n");
-#endif
-		return ERROR;
-	        }
-#endif
-
 	/* check service notification dependencies */
 	if(check_service_dependencies(svc,NOTIFICATION_DEPENDENCY)==DEPENDENCIES_FAILED){
 #ifdef DEBUG4
@@ -433,17 +439,6 @@ int check_service_notification_viability(service *svc, int type){
 	if(svc->current_state==STATE_OK)
 		return OK;
 
-#ifdef REMOVED_042103
-	/* don't send notifications if the host was down or unreachable at last service check */
-	/* removed this 4/21/03 because it doesn't make much sense... */
-	if(svc->host_problem_at_last_check==TRUE){
-#ifdef DEBUG4
-		printf("\tThe host associated with this service had problems at last check, so notifications won't be sent out!\n");
-#endif
-		return ERROR;
-	        }
-#endif
-
 	/* don't notify contacts about this service problem again if the notification interval is set to 0 */
 	if(svc->no_more_notifications==TRUE){
 #ifdef DEBUG4
@@ -452,14 +447,6 @@ int check_service_notification_viability(service *svc, int type){
 		return ERROR;
 	        }
 
-	/* if we couldn't find the host, return an error */
-	if(temp_host==NULL){
-#ifdef DEBUG4
-		printf("\tCouldn't find the host associated with this service, so we won't send a notification!\n");
-#endif
-		return ERROR;
-	        }
-
 	/* if the host is down or unreachable, don't notify contacts about service failures */
 	if(temp_host->current_state!=HOST_UP){
 #ifdef DEBUG4
@@ -1140,6 +1127,14 @@ int check_host_notification_viability(host *hst, int type){
 			return ERROR;
 	                }
 
+		/* don't send notifications during scheduled downtime */
+		if(hst->scheduled_downtime_depth>0){
+#ifdef DEBUG4
+			printf("\tWe shouldn't notify about FLAPPING events during scheduled downtime!\n");
+#endif
+			return ERROR;
+		        }
+
 		/* flapping viability test passed, so the notification can be sent out */
 		return OK;
 	        }
