@@ -2,15 +2,14 @@
  *
  * PERFDATA.C - Performance data routines for Nagios
  *
- * Copyright (c) 2000-2001 Ethan Galstad (nagios@nagios.org)
- * Last Modified:   06-12-2001
+ * Copyright (c) 2000-2004 Ethan Galstad (nagios@nagios.org)
+ * Last Modified:   11-29-2004
  *
  * License:
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,10 +25,10 @@
 
 /*********** COMMON HEADER FILES ***********/
 
-#include "../common/config.h"
-#include "../common/common.h"
-#include "../common/objects.h"
-#include "perfdata.h"
+#include "../include/config.h"
+#include "../include/common.h"
+#include "../include/objects.h"
+#include "../include/perfdata.h"
 
 
 /***** IMPLEMENTATION-SPECIFIC HEADER FILES *****/
@@ -37,9 +36,6 @@
 #ifdef USE_XPDDEFAULT
 #include "../xdata/xpddefault.h"
 #endif
-#ifdef USE_XPDFILE
-#include "../xdata/xpdfile.h"
-#endif
 
 
 extern int   process_performance_data;
@@ -57,10 +53,6 @@ int initialize_performance_data(char *config_file){
 	xpddefault_initialize_performance_data(config_file);
 #endif
 
-#ifdef USE_XPDFILE
-	xpdfile_initialize_performance_data(config_file);
-#endif
-
 	return OK;
         }
 
@@ -73,10 +65,6 @@ int cleanup_performance_data(char *config_file){
 	xpddefault_cleanup_performance_data(config_file);
 #endif
 
-#ifdef USE_XPDFILE
-	xpdfile_cleanup_performance_data(config_file);
-#endif
-
 	return OK;
         }
 
@@ -102,9 +90,6 @@ int update_service_performance_data(service *svc){
 #ifdef USE_XPDDEFAULT
 	xpddefault_update_service_performance_data(svc);
 #endif
-#ifdef USE_XPDFILE
-	xpdfile_update_service_performance_data(svc);
-#endif
 
 	return OK;
         }
@@ -112,7 +97,7 @@ int update_service_performance_data(service *svc){
 
 
 /* updates host performance data */
-int update_host_performance_data(host *hst, int state, int state_type){
+int update_host_performance_data(host *hst){
 
 	/* should we be processing performance data for anything? */
 	if(process_performance_data==FALSE)
@@ -124,10 +109,7 @@ int update_host_performance_data(host *hst, int state, int state_type){
 
 	/* process the performance data! */
 #ifdef USE_XPDDEFAULT
-	xpddefault_update_host_performance_data(hst,state,state_type);
-#endif
-#ifdef USE_XPDFILE
-	xpdfile_update_host_performance_data(hst,state,state_type);
+	xpddefault_update_host_performance_data(hst);
 #endif
 
 	return OK;
