@@ -2,8 +2,8 @@
  *
  * OUTAGES.C -  Nagios Network Outages CGI
  *
- * Copyright (c) 1999-2004 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 10-30-2004
+ * Copyright (c) 1999-2006 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 03-21-2006
  *
  * License:
  * 
@@ -206,17 +206,17 @@ void document_header(int use_stylesheet){
 	time_t current_time;
 	time_t expire_time;
 
-	printf("Cache-Control: no-store\n");
-	printf("Pragma: no-cache\n");
-	printf("Refresh: %d\n",refresh_rate);
+	printf("Cache-Control: no-store\r\n");
+	printf("Pragma: no-cache\r\n");
+	printf("Refresh: %d\r\n",refresh_rate);
 
 	time(&current_time);
 	get_time_string(&current_time,date_time,(int)sizeof(date_time),HTTP_DATE_TIME);
-	printf("Last-Modified: %s\n",date_time);
+	printf("Last-Modified: %s\r\n",date_time);
 
 	expire_time=(time_t)0L;
 	get_time_string(&expire_time,date_time,(int)sizeof(date_time),HTTP_DATE_TIME);
-	printf("Expires: %s\n",date_time);
+	printf("Expires: %s\r\n",date_time);
 
 	printf("Content-type: text/html\r\n\r\n");
 
