@@ -2,15 +2,14 @@
  *
  * STATUSWML.C -  Nagios Status CGI for WAP-enabled devices
  *
- * Copyright (c) 2001-2002 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 07-28-2002
+ * Copyright (c) 2001-2006 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 03-22-2006
  *
  * License:
  * 
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,15 +21,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *************************************************************************/
 
-#include "../common/config.h"
-#include "../common/locations.h"
-#include "../common/common.h"
-#include "../common/objects.h"
-#include "../common/statusdata.h"
+#include "../include/config.h"
+#include "../include/common.h"
+#include "../include/objects.h"
+#include "../include/statusdata.h"
 
-#include "cgiutils.h"
-#include "getcgi.h"
-#include "auth.h"
+#include "../include/cgiutils.h"
+#include "../include/getcgi.h"
+#include "../include/cgiauth.h"
 
 extern time_t          program_start;
 
@@ -38,9 +36,9 @@ extern char main_config_file[MAX_FILENAME_LENGTH];
 
 extern host *host_list;
 extern hostgroup *hostgroup_list;
+extern service *service_list;
 extern hoststatus *hoststatus_list;
 extern servicestatus *servicestatus_list;
-extern service *service_list;
 
 extern int      enable_notifications;
 extern int      execute_service_checks;
@@ -110,9 +108,9 @@ int main(void){
 	document_header();
 
 	/* read the CGI configuration file */
-	result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
+	result=read_cgi_config_file(get_cgi_config_location());
 	if(result==ERROR){
-		printf("<P>Error: Could not open CGI configuration file '%s' for reading!</P>\n",DEFAULT_CGI_CONFIG_FILE);
+		printf("<P>Error: Could not open CGI configuration file '%s' for reading!</P>\n",get_cgi_config_location());
 		document_footer();
 		return ERROR;
 	        }
@@ -134,7 +132,7 @@ int main(void){
                 }
 
 	/* read all status data */
-	result=read_all_status_data(DEFAULT_CGI_CONFIG_FILE,READ_ALL_STATUS_DATA);
+	result=read_all_status_data(get_cgi_config_location(),READ_ALL_STATUS_DATA);
 	if(result==ERROR){
 		printf("<P>Error: Could not read host and service status information!</P>\n");
 		document_footer();
@@ -185,17 +183,17 @@ void document_header(void){
 
 	time(&current_time);
 
-	printf("Cache-Control: no-store\n");
-	printf("Pragma: no-cache\n");
+	printf("Cache-Control: no-store\r\n");
+	printf("Pragma: no-cache\r\n");
 
 	get_time_string(&current_time,date_time,(int)sizeof(date_time),HTTP_DATE_TIME);
-	printf("Last-Modified: %s\n",date_time);
+	printf("Last-Modified: %s\r\n",date_time);
 
 	expire_time=(time_t)0L;
 	get_time_string(&expire_time,date_time,(int)sizeof(date_time),HTTP_DATE_TIME);
-	printf("Expires: %s\n",date_time);
+	printf("Expires: %s\r\n",date_time);
 
-	printf("Content-type: text/vnd.wap.wml\n\n");
+	printf("Content-type: text/vnd.wap.wml\r\n\r\n");
 
 	printf("<?xml version=\"1.0\"?>\n");
 	printf("<!DOCTYPE wml PUBLIC \"-//WAPFORUM//DTD WML 1.1//EN\" \"http://www.wapforum.org/DTD/wml_1.1.xml\">\n");
@@ -609,6 +607,7 @@ void display_quick_stats(void){
 /* displays hostgroup status overview */
 void display_hostgroup_overview(void){
 	hostgroup *temp_hostgroup;
+	hostgroupmember *temp_member;
 	host *temp_host;
 	hoststatus *temp_hoststatus;
 
@@ -633,9 +632,10 @@ void display_hostgroup_overview(void){
 		printf("<table columns='2' align='LL'>\n");
 
 		/* check all hosts in this hostgroup */
-		for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
+		for(temp_member=temp_hostgroup->members;temp_member!=NULL;temp_member=temp_member->next){
 
-			if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
+			temp_host=find_host(temp_member->host_name);
+			if(temp_host==NULL)
 				continue;
 
 			if(is_host_member_of_hostgroup(temp_hostgroup,temp_host)==FALSE)
@@ -678,6 +678,7 @@ void display_hostgroup_overview(void){
 /* displays hostgroup status summary */
 void display_hostgroup_summary(void){
 	hostgroup *temp_hostgroup;
+	hostgroupmember *temp_member;
 	host *temp_host;
 	hoststatus *temp_hoststatus;
 	service *temp_service;
@@ -725,9 +726,10 @@ void display_hostgroup_summary(void){
 		services_critical=0;
 
 		/* check all hosts in this hostgroup */
-		for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
+		for(temp_member=temp_hostgroup->members;temp_member!=NULL;temp_member=temp_member->next){
 
-			if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
+			temp_host=find_host(temp_member->host_name);
+			if(temp_host==NULL)
 				continue;
 
 			if(is_host_member_of_hostgroup(temp_hostgroup,temp_host)==FALSE)
@@ -749,10 +751,10 @@ void display_hostgroup_summary(void){
 			/* check all services on this host */
 			for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
 
-				if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
+				if(strcmp(temp_service->host_name,temp_host->name))
 					continue;
 
-				if(strcmp(temp_service->host_name,temp_host->name))
+				if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
 					continue;
 
 				temp_servicestatus=find_servicestatus(temp_service->host_name,temp_service->description);
@@ -822,7 +824,7 @@ void display_hostgroup_summary(void){
 	printf("</card>\n");
 
 	return;
-        }
+        } 
 
 
 
@@ -846,7 +848,7 @@ void display_host(void){
 	printf("<b>Host '%s'</b><br/>\n",host_name);
 
 	/* find the host */
-	temp_host=find_host(host_name,NULL);
+	temp_host=find_host(host_name);
 	temp_hoststatus=find_hoststatus(host_name);
 	if(temp_host==NULL || temp_hoststatus==NULL){
 
@@ -881,7 +883,7 @@ void display_host(void){
 		printf("?");
 	printf("</td></tr>\n");
 
-	printf("<tr><td>Info:</td><td>%s</td></tr>\n",temp_hoststatus->information);
+	printf("<tr><td>Info:</td><td>%s</td></tr>\n",temp_hoststatus->plugin_output);
 
 	get_time_string(&temp_hoststatus->last_check,last_check,sizeof(last_check)-1,SHORT_DATE_TIME);
 	printf("<tr><td>Last Check:</td><td>%s</td></tr>\n",last_check);
@@ -1001,10 +1003,10 @@ void display_host_services(void){
 	/* check all services */
 	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
 
-		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
+		if(strcmp(temp_service->host_name,host_name))
 			continue;
 
-		if(strcmp(temp_service->host_name,host_name))
+		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
 			continue;
 
 		temp_servicestatus=find_servicestatus(temp_service->host_name,temp_service->description);
@@ -1012,7 +1014,7 @@ void display_host_services(void){
 			continue;
 
 		printf("<tr><td><anchor title='%s'>",temp_service->description);
-		if(temp_servicestatus->status==SERVICE_OK || temp_servicestatus->status==SERVICE_RECOVERY)
+		if(temp_servicestatus->status==SERVICE_OK)
 			printf("OK");
 		else if(temp_servicestatus->status==SERVICE_PENDING)
 			printf("PND");
@@ -1060,7 +1062,7 @@ void display_service(void){
 	printf("<b>Service '%s' on host '%s'</b><br/>\n",service_desc,host_name);
 
 	/* find the service */
-	temp_service=find_service(host_name,service_desc,NULL);
+	temp_service=find_service(host_name,service_desc);
 	temp_servicestatus=find_servicestatus(host_name,service_desc);
 	if(temp_service==NULL || temp_servicestatus==NULL){
 
@@ -1083,7 +1085,7 @@ void display_service(void){
 	printf("<table columns='2' align='LL'>\n");
 
 	printf("<tr><td>Status:</td><td>");
-	if(temp_servicestatus->status==SERVICE_OK || temp_servicestatus->status==SERVICE_RECOVERY)
+	if(temp_servicestatus->status==SERVICE_OK)
 		printf("OK");
 	else if(temp_servicestatus->status==SERVICE_PENDING)
 		printf("PENDING");
@@ -1097,7 +1099,7 @@ void display_service(void){
 		printf("?");
 	printf("</td></tr>\n");
 
-	printf("<tr><td>Info:</td><td>%s</td></tr>\n",temp_servicestatus->information);
+	printf("<tr><td>Info:</td><td>%s</td></tr>\n",temp_servicestatus->plugin_output);
 
 	get_time_string(&temp_servicestatus->last_check,last_check,sizeof(last_check)-1,SHORT_DATE_TIME);
 	printf("<tr><td>Last Check:</td><td>%s</td></tr>\n",last_check);
@@ -1147,14 +1149,14 @@ void display_service(void){
 	printf("<p align='center' mode='nowrap'>\n");
 	printf("<b>Service Commands</b><br/>\n");
 
-	if(temp_servicestatus->status!=SERVICE_OK && temp_servicestatus->status!=SERVICE_RECOVERY && temp_servicestatus->status!=SERVICE_PENDING)
+	if(temp_servicestatus->status!=SERVICE_OK && temp_servicestatus->status!=SERVICE_PENDING)
 		printf("<b><anchor title='Acknowledge Problem'>Acknowledge Problem<go href='#card3'/></anchor></b>\n");
 
 	if(temp_servicestatus->checks_enabled==FALSE)
 		printf("<b><anchor title='Enable Checks'>Enable Checks<go href='%s' method='post'><postfield name='host' value='%s'/><postfield name='service' value='%s'/><postfield name='cmd_typ' value='%d'/><postfield name='cmd_mod' value='%d'/><postfield name='content' value='wml'/></go></anchor></b><br/>\n",COMMAND_CGI,host_name,service_desc,CMD_ENABLE_SVC_CHECK,CMDMODE_COMMIT);
 	else{
 		printf("<b><anchor title='Disable Checks'>Disable Checks<go href='%s' method='post'><postfield name='host' value='%s'/><postfield name='service' value='%s'/><postfield name='cmd_typ' value='%d'/><postfield name='cmd_mod' value='%d'/><postfield name='content' value='wml'/></go></anchor></b><br/>\n",COMMAND_CGI,host_name,service_desc,CMD_DISABLE_SVC_CHECK,CMDMODE_COMMIT);
-		printf("<b><anchor title='Schedule Immediate Check'>Schedule Immediate Check<go href='%s' method='post'><postfield name='host' value='%s'/><postfield name='service' value='%s'/><postfield name='cmd_typ' value='%d'/><postfield name='cmd_mod' value='%d'/><postfield name='content' value='wml'/></go></anchor></b><br/>\n",COMMAND_CGI,host_name,service_desc,CMD_IMMEDIATE_SVC_CHECK,CMDMODE_COMMIT);
+		printf("<b><anchor title='Schedule Immediate Check'>Schedule Immediate Check<go href='%s' method='post'><postfield name='host' value='%s'/><postfield name='service' value='%s'/><postfield name='cmd_typ' value='%d'/><postfield name='cmd_mod' value='%d'/><postfield name='content' value='wml'/></go></anchor></b><br/>\n",COMMAND_CGI,host_name,service_desc,CMD_SCHEDULE_SVC_CHECK,CMDMODE_COMMIT);
 	        }
 
 	if(temp_servicestatus->notifications_enabled==FALSE)
@@ -1381,7 +1383,7 @@ void display_problems(void){
 	/* check all hosts */
 	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){
 
-		temp_host=find_host(temp_hoststatus->host_name,NULL);
+		temp_host=find_host(temp_hoststatus->host_name);
 		if(temp_host==NULL)
 			continue;
 
@@ -1394,8 +1396,6 @@ void display_problems(void){
 		if(display_type==DISPLAY_UNHANDLED_PROBLEMS){
 			if(temp_hoststatus->problem_has_been_acknowledged==TRUE)
 				continue;
-			if(temp_hoststatus->checks_enabled==FALSE)
-				continue;
 			if(temp_hoststatus->notifications_enabled==FALSE)
 				continue;
 			if(temp_hoststatus->scheduled_downtime_depth>0)
@@ -1430,25 +1430,29 @@ void display_problems(void){
 	/* check all services */
 	for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){
 		
-		temp_service=find_service(temp_servicestatus->host_name,temp_servicestatus->description,NULL);
+		temp_service=find_service(temp_servicestatus->host_name,temp_servicestatus->description);
 		if(temp_service==NULL)
 			continue;
 
 		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
 			continue;
 
-		if(temp_servicestatus->status==SERVICE_OK || temp_servicestatus->status==SERVICE_RECOVERY || temp_servicestatus->status==SERVICE_PENDING)
+		if(temp_servicestatus->status==SERVICE_OK || temp_servicestatus->status==SERVICE_PENDING)
 			continue;
 
 		if(display_type==DISPLAY_UNHANDLED_PROBLEMS){
 			if(temp_servicestatus->problem_has_been_acknowledged==TRUE)
 				continue;
-			if(temp_servicestatus->checks_enabled==FALSE)
-				continue;
 			if(temp_servicestatus->notifications_enabled==FALSE)
 				continue;
 			if(temp_servicestatus->scheduled_downtime_depth>0)
 				continue;
+			if((temp_hoststatus=find_hoststatus(temp_service->host_name))){
+				if(temp_hoststatus->scheduled_downtime_depth>0)
+					continue;
+				if(temp_hoststatus->problem_has_been_acknowledged==TRUE)
+					continue;
+				}
 		        }
 
 		total_service_problems++;
