@@ -2,8 +2,8 @@
  *
  * SUMMARY.C -  Nagios Alert Summary CGI
  *
- * Copyright (c) 2002-2005 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 05-22-2005
+ * Copyright (c) 2002-2006 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 03-21-2006
  *
  * License:
  * 
@@ -685,21 +685,21 @@ void document_header(int use_stylesheet){
 	time_t current_time;
 	time_t expire_time;
 
-	printf("Cache-Control: no-store\n");
-	printf("Pragma: no-cache\n");
+	printf("Cache-Control: no-store\r\n");
+	printf("Pragma: no-cache\r\n");
 
 	time(&current_time);
 	get_time_string(&current_time,date_time,sizeof(date_time),HTTP_DATE_TIME);
-	printf("Last-Modified: %s\n",date_time);
+	printf("Last-Modified: %s\r\n",date_time);
 
 	expire_time=(time_t)0;
 	get_time_string(&expire_time,date_time,sizeof(date_time),HTTP_DATE_TIME);
-	printf("Expires: %s\n",date_time);
+	printf("Expires: %s\r\n",date_time);
 
 	if(output_format==HTML_OUTPUT)
-		printf("Content-type: text/html\n\n");
+		printf("Content-type: text/html\r\n\r\n");
 	else{
-		printf("Content-type: text/plain\n\n");
+		printf("Content-type: text/plain\r\n\r\n");
 		return;
 	        }
 
