@@ -2,15 +2,14 @@
  *
  * DOWNTIME.C - Scheduled downtime functions for Nagios
  *
- * Copyright (c) 2000-2004 Ethan Galstad (nagios@nagios.org)
- * Last Modified:   10-24-2004
+ * Copyright (c) 2000-2005 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 06-12-2005
  *
  * License:
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,11 +22,12 @@
  *
  *****************************************************************************/
 
-#include "config.h"
-#include "common.h"
-#include "comments.h"
-#include "downtime.h"
-#include "objects.h"
+#include "../include/config.h"
+#include "../include/common.h"
+#include "../include/comments.h"
+#include "../include/downtime.h"
+#include "../include/objects.h"
+#include "../include/statusdata.h"
 
 /***** IMPLEMENTATION-SPECIFIC INCLUDES *****/
 
@@ -36,12 +36,12 @@
 #endif
 
 #ifdef NSCORE
-#include "statusdata.h"
-#include "../base/nagios.h"
+#include "../include/nagios.h"
+#include "../include/broker.h"
 #endif
 
 #ifdef NSCGI
-#include "../cgi/cgiutils.h"
+#include "../include/cgiutils.h"
 #endif
 
 
@@ -60,7 +60,7 @@ extern timed_event *event_list_high;
 /******************************************************************/
 
 
-/* initalizes scheduled downtime data */
+/* initializes scheduled downtime data */
 int initialize_downtime_data(char *config_file){
 	int result;
 
@@ -68,9 +68,6 @@ int initialize_downtime_data(char *config_file){
 #ifdef USE_XDDDEFAULT
 	result=xdddefault_initialize_downtime_data(config_file);
 #endif
-#ifdef USE_XDDDB
-	result=xdddb_initialize_downtime_data(config_file);
-#endif
 
 	return result;
         }
@@ -84,9 +81,6 @@ int cleanup_downtime_data(char *config_file){
 #ifdef USE_XDDDEFAULT
 	result=xdddefault_cleanup_downtime_data(config_file);
 #endif
-#ifdef USE_XDDDB
-	result=xdddb_cleanup_downtime_data(config_file);
-#endif
 
 	/* free memory allocated to downtime data */
 	free_downtime_data();
@@ -100,62 +94,53 @@ int cleanup_downtime_data(char *config_file){
 /******************************************************************/
 
 /* schedules a host or service downtime */
-int schedule_downtime(int type, char *host_name, char *service_description, time_t entry_time, char *author, char *comment, time_t start_time, time_t end_time, int fixed, unsigned long duration){
-	int downtime_id;
+int schedule_downtime(int type, char *host_name, char *service_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long *new_downtime_id){
+	unsigned long downtime_id;
 
 	/* don't add old or invalid downtimes */
 	if(start_time>=end_time || end_time<=time(NULL))
 		return ERROR;
 
-	/* add downtime entry to log */
-	save_downtime(type,host_name,service_description,entry_time,author,comment,start_time,end_time,fixed,duration,&downtime_id);
-
-	/* add to list in memory */
-	add_downtime(type,host_name,service_description,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+	/* add a new downtime entry */
+	add_new_downtime(type,host_name,service_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,&downtime_id);
 
 	/* register the scheduled downtime */
 	register_downtime(type,downtime_id);
 
+	/* return downtime id */
+	if(new_downtime_id!=NULL)
+		*new_downtime_id=downtime_id;
+
 	return OK;
         }
 
 
 /* unschedules a host or service downtime */
-int unschedule_downtime(int type,int downtime_id){
+int unschedule_downtime(int type,unsigned long downtime_id){
 	scheduled_downtime *temp_downtime;
-	scheduled_downtime *temp2_downtime;
 	scheduled_downtime *event_downtime;
 	host *hst=NULL;
 	service *svc=NULL;
 	timed_event *temp_event;
 	char temp_buffer[MAX_INPUT_BUFFER];
+#ifdef USE_EVENT_BROKER
+	int attr;
+#endif
 	
-	/* delete downtime entry from log */
-	delete_downtime(type,downtime_id);
 
 	/* find the downtime entry in the list in memory */
-	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
-		if(temp_downtime->type!=type)
-			continue;
-		if(temp_downtime->downtime_id==downtime_id)
-			break;
-	        }
+	temp_downtime=find_downtime(type,downtime_id);
 	if(temp_downtime==NULL)
 		return ERROR;
 
-
-	/* delete the comment we added */
-	delete_comment((temp_downtime->type==HOST_DOWNTIME)?HOST_COMMENT:SERVICE_COMMENT,temp_downtime->comment_id);
-
-
 	/* find the host or service associated with this downtime */
 	if(temp_downtime->type==HOST_DOWNTIME){
-		hst=find_host(temp_downtime->host_name,NULL);
+		hst=find_host(temp_downtime->host_name);
 		if(hst==NULL)
 			return ERROR;
 	        }
 	else{
-		svc=find_service(temp_downtime->host_name,temp_downtime->service_description,NULL);
+		svc=find_service(temp_downtime->host_name,temp_downtime->service_description);
 		if(svc==NULL)
 			return ERROR;
 	        }
@@ -171,6 +156,12 @@ int unschedule_downtime(int type,int downtime_id){
 	/* decrement the downtime depth variable and update status data if necessary */
 	if(temp_downtime->is_in_effect==TRUE){
 
+#ifdef USE_EVENT_BROKER
+		/* send data to event broker */
+		attr=NEBATTR_DOWNTIME_STOP_CANCELLED;
+		broker_downtime_data(NEBTYPE_DOWNTIME_STOP,NEBFLAG_NONE,attr,temp_downtime->type,temp_downtime->host_name,temp_downtime->service_description,temp_downtime->entry_time,temp_downtime->author,temp_downtime->comment,temp_downtime->start_time,temp_downtime->end_time,temp_downtime->fixed,temp_downtime->triggered_by,temp_downtime->duration,temp_downtime->downtime_id,NULL);
+#endif
+
 		if(temp_downtime->type==HOST_DOWNTIME){
 
 			hst->scheduled_downtime_depth--;
@@ -180,7 +171,7 @@ int unschedule_downtime(int type,int downtime_id){
 			if(hst->scheduled_downtime_depth==0){
 				snprintf(temp_buffer,sizeof(temp_buffer),"HOST DOWNTIME ALERT: %s;CANCELLED; Scheduled downtime for host has been cancelled.\n",hst->name);
 				temp_buffer[sizeof(temp_buffer)-1]='\x0';
-				write_to_logs_and_console(temp_buffer,NSLOG_INFO_MESSAGE,FALSE);
+				write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
 			        }
 		        }
 
@@ -193,18 +184,17 @@ int unschedule_downtime(int type,int downtime_id){
 			if(svc->scheduled_downtime_depth==0){
 				snprintf(temp_buffer,sizeof(temp_buffer),"SERVICE DOWNTIME ALERT: %s;%s;CANCELLED; Scheduled downtime for service has been cancelled.\n",svc->host_name,svc->description);
 				temp_buffer[sizeof(temp_buffer)-1]='\x0';
-				write_to_logs_and_console(temp_buffer,NSLOG_INFO_MESSAGE,FALSE);
+				write_to_all_logs(temp_buffer,NSLOG_INFO_MESSAGE);
 			        }
 		        }
 	        }
 
-
 	/* remove scheduled entry from event queue */
 	for(temp_event=event_list_high;temp_event!=NULL;temp_event=temp_event->next){
 		if(temp_event->event_type!=EVENT_SCHEDULED_DOWNTIME)
 			continue;
 		event_downtime=(scheduled_downtime *)temp_event->event_data;
-		if(event_downtime->type!=type)
+		if(event_downtime->type!=temp_downtime->type)
 			continue;
 		if(event_downtime->downtime_id==downtime_id)
 			break;
@@ -212,22 +202,16 @@ int unschedule_downtime(int type,int downtime_id){
 	if(temp_event!=NULL)
 		remove_event(temp_event,&event_list_high);
 
+	/* delete downtime entry */
+	if(temp_downtime->type==HOST_DOWNTIME)
+		delete_host_downtime(downtime_id);
+	else
+		delete_service_downtime(downtime_id);
 
-	/* delete downtime entry from list in memory */
-	if(temp_downtime!=NULL){
-		if(scheduled_downtime_list==temp_downtime)
-			scheduled_downtime_list=temp_downtime->next;
-		else{
-			for(temp2_downtime=scheduled_downtime_list;temp2_downtime!=NULL;temp2_downtime=temp2_downtime->next){
-				if(temp2_downtime->next==temp_downtime)
-					temp2_downtime->next=temp_downtime->next;
-			        }
-		        }
-		free(temp_downtime->host_name);
-		free(temp_downtime->service_description);
-		free(temp_downtime->comment);
-		free(temp_downtime->author);
-		free(temp_downtime);
+	/* unschedule all downtime entries that were triggered by this one */
+	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
+		if(temp_downtime->triggered_by==downtime_id)
+			unschedule_downtime(ANY_DOWNTIME,downtime_id);
 	        }
 
 	return OK;
@@ -236,35 +220,30 @@ int unschedule_downtime(int type,int downtime_id){
 
 
 /* registers scheduled downtime (schedules it, adds comments, etc.) */
-int register_downtime(int type, int downtime_id){
+int register_downtime(int type, unsigned long downtime_id){
  	char temp_buffer[MAX_INPUT_BUFFER];
 	char start_time_string[MAX_DATETIME_LENGTH];
 	char end_time_string[MAX_DATETIME_LENGTH];
 	scheduled_downtime *temp_downtime;
-	timed_event *new_event;
 	host *hst=NULL;
 	service *svc=NULL;
+	char *type_string="";
 	int hours;
 	int minutes;
 
 	/* find the downtime entry in memory */
-	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
-		if(temp_downtime->type!=type)
-			continue;
-		if(temp_downtime->downtime_id==downtime_id)
-			break;
-	        }
+	temp_downtime=find_downtime(type,downtime_id);
 	if(temp_downtime==NULL)
 		return ERROR;
 
 	/* find the host or service associated with this downtime */
 	if(temp_downtime->type==HOST_DOWNTIME){
-		hst=find_host(temp_downtime->host_name,NULL);
+		hst=find_host(temp_downtime->host_name);
 		if(hst==NULL)
 			return ERROR;
 	        }
 	else{
-		svc=find_service(temp_downtime->host_name,temp_downtime->service_description,NULL);
+		svc=find_service(temp_downtime->host_name,temp_downtime->service_description);
 		if(svc==NULL)
 			return ERROR;
 	        }
@@ -274,41 +253,31 @@ int register_downtime(int type, int downtime_id){
 	get_datetime_string(&(temp_downtime->end_time),end_time_string,MAX_DATETIME_LENGTH,SHORT_DATE_TIME);
 	hours=temp_downtime->duration/3600;
 	minutes=((temp_downtime->duration-(hours*3600))/60);
+	if(temp_downtime->type==HOST_DOWNTIME)
+		type_string="host";
+	else
+		type_string="service";
 	if(temp_downtime->fixed==TRUE){
-		if(temp_downtime->type==HOST_DOWNTIME)
-			snprintf(temp_buffer,sizeof(temp_buffer)-1,"This host has been scheduled for downtime from %s to %s.  Notifications for the host will not be sent out during that time period.",start_time_string,end_time_string);
-		else
-			snprintf(temp_buffer,sizeof(temp_buffer)-1,"This service has been scheduled for downtime from %s to %s.  Notifications for the service will not be sent out during that time period.",start_time_string,end_time_string);
+		snprintf(temp_buffer,sizeof(temp_buffer)-1,"This %s has been scheduled for fixed downtime from %s to %s.  Notifications for the %s will not be sent out during that time period.",type_string,start_time_string,end_time_string,type_string);
 	        }
 	else{
-		if(temp_downtime->type==HOST_DOWNTIME)
-			snprintf(temp_buffer,sizeof(temp_buffer)-1,"This host has been scheduled for downtime starting between %s and %s for a period of %d hours and %d minutes.  Notifications for the host will not be sent out during that time period.",start_time_string,end_time_string,hours,minutes);
-		else
-			snprintf(temp_buffer,sizeof(temp_buffer)-1,"This service has been scheduled for downtime starting between %s and %s for a period of %d hours and %d minutes.  Notifications for the service will not be sent out during that time period.",start_time_string,end_time_string,hours,minutes);
+		snprintf(temp_buffer,sizeof(temp_buffer)-1,"This %s has been scheduled for flexible downtime starting between %s and %s and lasting for a period of %d hours and %d minutes.  Notifications for the %s will not be sent out during that time period.",type_string,start_time_string,end_time_string,hours,minutes,type_string);
 	        }
 	temp_buffer[sizeof(temp_buffer)-1]='\x0';
 
 
 	/* add a non-persistent comment to the host or service regarding the scheduled outage */
 	if(temp_downtime->type==SERVICE_DOWNTIME)
-		save_comment(SERVICE_COMMENT,svc->host_name,svc->description,time(NULL),"(Nagios Process)",temp_buffer,0,&(temp_downtime->comment_id));
+		add_new_comment(SERVICE_COMMENT,DOWNTIME_COMMENT,svc->host_name,svc->description,time(NULL),"(Nagios Process)",temp_buffer,0,COMMENTSOURCE_INTERNAL,FALSE,(time_t)0,&(temp_downtime->comment_id));
 	else
-		save_comment(HOST_COMMENT,hst->name,NULL,time(NULL),"(Nagios Process)",temp_buffer,0,&(temp_downtime->comment_id));
+		add_new_comment(HOST_COMMENT,DOWNTIME_COMMENT,hst->name,NULL,time(NULL),"(Nagios Process)",temp_buffer,0,COMMENTSOURCE_INTERNAL,FALSE,(time_t)0,&(temp_downtime->comment_id));
 
 
 	/*** SCHEDULE DOWNTIME - FLEXIBLE (NON-FIXED) DOWNTIME IS HANDLED AT A LATER POINT ***/
 
-	/* allocate memory for a new event queue item */
-	new_event=(timed_event *)malloc(sizeof(timed_event));
-	if(new_event==NULL)
-		return ERROR;
-
-	/* place the new event in the event queue */
-	new_event->event_type=EVENT_SCHEDULED_DOWNTIME;
-	new_event->event_data=(void *)temp_downtime;
-	new_event->run_time=temp_downtime->start_time;
-	new_event->recurring=FALSE;
-	schedule_event(new_event,&event_list_high);
+	/* only non-triggered downtime is scheduled... */
+	if(temp_downtime->triggered_by==0)
+		schedule_new_event(EVENT_SCHEDULED_DOWNTIME,TRUE,temp_downtime->start_time,FALSE,0,NULL,FALSE,(void *)temp_downtime,NULL);
 
 	return OK;
         }
@@ -316,11 +285,14 @@ int register_downtime(int type, int downtime_id){
 
 /* handles scheduled host or service downtime */
 int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
-	scheduled_downtime *temp2_downtime;
  	char buffer[MAX_INPUT_BUFFER];
-	timed_event *new_event;
+	scheduled_downtime *this_downtime;
 	host *hst=NULL;
 	service *svc=NULL;
+	time_t event_time;
+#ifdef USE_EVENT_BROKER
+	int attr;
+#endif
 
 #ifdef DEBUG0
         printf("handle_scheduled_downtime() start\n");
@@ -331,12 +303,12 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 
 	/* find the host or service associated with this downtime */
 	if(temp_downtime->type==HOST_DOWNTIME){
-		hst=find_host(temp_downtime->host_name,NULL);
+		hst=find_host(temp_downtime->host_name);
 		if(hst==NULL)
 			return ERROR;
 	        }
 	else{
-		svc=find_service(temp_downtime->host_name,temp_downtime->service_description,NULL);
+		svc=find_service(temp_downtime->host_name,temp_downtime->service_description);
 		if(svc==NULL)
 			return ERROR;
 	        }
@@ -349,7 +321,7 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 		if(temp_downtime->start_flex_downtime==FALSE){
 
 			/* host is up or service is ok, so we don't really do anything right now */
-			if((temp_downtime->type==HOST_DOWNTIME && hst->status==HOST_UP) || (temp_downtime->type==SERVICE_DOWNTIME && svc->current_state==STATE_OK)){
+			if((temp_downtime->type==HOST_DOWNTIME && hst->current_state==HOST_UP) || (temp_downtime->type==SERVICE_DOWNTIME && svc->current_state==STATE_OK)){
 
 				/* increment pending flex downtime counter */
 				if(temp_downtime->type==HOST_DOWNTIME)
@@ -360,17 +332,7 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 
 				/*** SINCE THE FLEX DOWNTIME MAY NEVER START, WE HAVE TO PROVIDE A WAY OF EXPIRING UNUSED DOWNTIME... ***/
 
-				/* allocate memory for a new event queue item */
-				new_event=(timed_event *)malloc(sizeof(timed_event));
-				if(new_event==NULL)
-					return ERROR;
-
-				/* place the new event in the event queue */
-				new_event->event_type=EVENT_EXPIRE_DOWNTIME;
-				new_event->event_data=NULL;
-				new_event->run_time=(time_t)(temp_downtime->end_time+1);
-				new_event->recurring=FALSE;
-				schedule_event(new_event,&event_list_high);
+				schedule_new_event(EVENT_EXPIRE_DOWNTIME,TRUE,(temp_downtime->end_time+1),FALSE,0,NULL,FALSE,NULL,NULL);
 
 				return OK;
 			        }
@@ -380,6 +342,12 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 	/* have we come to the end of the scheduled downtime? */
 	if(temp_downtime->is_in_effect==TRUE){
 
+#ifdef USE_EVENT_BROKER
+		/* send data to event broker */
+		attr=NEBATTR_DOWNTIME_STOP_NORMAL;
+		broker_downtime_data(NEBTYPE_DOWNTIME_STOP,NEBFLAG_NONE,attr,temp_downtime->type,temp_downtime->host_name,temp_downtime->service_description,temp_downtime->entry_time,temp_downtime->author,temp_downtime->comment,temp_downtime->start_time,temp_downtime->end_time,temp_downtime->fixed,temp_downtime->triggered_by,temp_downtime->duration,temp_downtime->downtime_id,NULL);
+#endif
+
 		/* decrement the downtime depth variable */
 		if(temp_downtime->type==HOST_DOWNTIME)
 			hst->scheduled_downtime_depth--;
@@ -391,7 +359,7 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 			/* log a notice - this one is parsed by the history CGI */
 			snprintf(buffer,sizeof(buffer),"HOST DOWNTIME ALERT: %s;STOPPED; Host has exited from a period of scheduled downtime",hst->name);
 			buffer[sizeof(buffer)-1]='\x0';
-			write_to_logs_and_console(buffer,NSLOG_INFO_MESSAGE,FALSE);
+			write_to_all_logs(buffer,NSLOG_INFO_MESSAGE);
 		        }
 
 		else if(temp_downtime->type==SERVICE_DOWNTIME && svc->scheduled_downtime_depth==0){
@@ -399,7 +367,7 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 			/* log a notice - this one is parsed by the history CGI */
 			snprintf(buffer,sizeof(buffer),"SERVICE DOWNTIME ALERT: %s;%s;STOPPED; Service has exited from a period of scheduled downtime",svc->host_name,svc->description);
 			buffer[sizeof(buffer)-1]='\x0';
-			write_to_logs_and_console(buffer,NSLOG_INFO_MESSAGE,FALSE);
+			write_to_all_logs(buffer,NSLOG_INFO_MESSAGE);
 		        }
 
 
@@ -409,13 +377,7 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 		else
 			update_service_status(svc,FALSE);
 
-		/* delete the comment we added */
-		delete_comment((temp_downtime->type==HOST_DOWNTIME)?HOST_COMMENT:SERVICE_COMMENT,temp_downtime->comment_id);
-
-		/* delete downtime entry from the log */
-		delete_downtime(temp_downtime->type,temp_downtime->downtime_id);
-
-		/* decrement pending flex downtime if necessary (there really isn't any reason these counters are necessary, so remove them in 2.0 - 11/16/02 EG) ... */
+		/* decrement pending flex downtime if necessary */
 		if(temp_downtime->fixed==FALSE && temp_downtime->incremented_pending_downtime==TRUE){
 			if(temp_downtime->type==HOST_DOWNTIME){
 				if(hst->pending_flex_downtime>0)
@@ -427,34 +389,33 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 			        }
 		        }
 
-		/* delete downtime entry from list in memory */
-		if(temp_downtime!=NULL){
-			if(scheduled_downtime_list==temp_downtime)
-				scheduled_downtime_list=temp_downtime->next;
-			else{
-				for(temp2_downtime=scheduled_downtime_list;temp2_downtime!=NULL;temp2_downtime=temp2_downtime->next){
-					if(temp2_downtime->next==temp_downtime)
-						temp2_downtime->next=temp_downtime->next;
-			                }
-		                }
-			free(temp_downtime->host_name);
-			free(temp_downtime->service_description);
-			free(temp_downtime->comment);
-			free(temp_downtime->author);
-			free(temp_downtime);
-	                }
+		/* delete downtime entry from the log */
+		if(temp_downtime->type==HOST_DOWNTIME)
+			delete_host_downtime(temp_downtime->downtime_id);
+		else
+			delete_service_downtime(temp_downtime->downtime_id);
 
+		/* handle (stop) downtime that is triggered by this one */
+		for(this_downtime=scheduled_downtime_list;this_downtime!=NULL;this_downtime=this_downtime->next){
+			if(this_downtime->triggered_by==temp_downtime->downtime_id)
+				handle_scheduled_downtime(this_downtime);
+		        }
 	        }
 
 	/* else we are just starting the scheduled downtime */
 	else{
 
+#ifdef USE_EVENT_BROKER
+		/* send data to event broker */
+		broker_downtime_data(NEBTYPE_DOWNTIME_START,NEBFLAG_NONE,NEBATTR_NONE,temp_downtime->type,temp_downtime->host_name,temp_downtime->service_description,temp_downtime->entry_time,temp_downtime->author,temp_downtime->comment,temp_downtime->start_time,temp_downtime->end_time,temp_downtime->fixed,temp_downtime->triggered_by,temp_downtime->duration,temp_downtime->downtime_id,NULL);
+#endif
+
 		if(temp_downtime->type==HOST_DOWNTIME && hst->scheduled_downtime_depth==0){
 
 			/* log a notice - this one is parsed by the history CGI */
 			snprintf(buffer,sizeof(buffer),"HOST DOWNTIME ALERT: %s;STARTED; Host has entered a period of scheduled downtime",hst->name);
 			buffer[sizeof(buffer)-1]='\x0';
-			write_to_logs_and_console(buffer,NSLOG_INFO_MESSAGE,FALSE);
+			write_to_all_logs(buffer,NSLOG_INFO_MESSAGE);
 		        }
 
 		else if(temp_downtime->type==SERVICE_DOWNTIME && svc->scheduled_downtime_depth==0){
@@ -462,7 +423,7 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 			/* log a notice - this one is parsed by the history CGI */
 			snprintf(buffer,sizeof(buffer),"SERVICE DOWNTIME ALERT: %s;%s;STARTED; Service has entered a period of scheduled downtime",svc->host_name,svc->description);
 			buffer[sizeof(buffer)-1]='\x0';
-			write_to_logs_and_console(buffer,NSLOG_INFO_MESSAGE,FALSE);
+			write_to_all_logs(buffer,NSLOG_INFO_MESSAGE);
 		        }
 
 		/* increment the downtime depth variable */
@@ -480,20 +441,18 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 		else
 			update_service_status(svc,FALSE);
 
-		/* allocate memory for a new event queue item */
-		new_event=(timed_event *)malloc(sizeof(timed_event));
-		if(new_event==NULL)
-			return ERROR;
-
-		/* place the new event in the event queue */
-		new_event->event_type=EVENT_SCHEDULED_DOWNTIME;
-		new_event->event_data=(void *)temp_downtime;
+		/* schedule an event */
 		if(temp_downtime->fixed==FALSE)
-			new_event->run_time=(time_t)((unsigned long)time(NULL)+temp_downtime->duration);
+			event_time=(time_t)((unsigned long)time(NULL)+temp_downtime->duration);
 		else
-			new_event->run_time=temp_downtime->end_time;
-		new_event->recurring=FALSE;
-		schedule_event(new_event,&event_list_high);
+			event_time=temp_downtime->end_time;
+		schedule_new_event(EVENT_SCHEDULED_DOWNTIME,TRUE,event_time,FALSE,0,NULL,FALSE,(void *)temp_downtime,NULL);
+
+		/* handle (start) downtime that is triggered by this one */
+		for(this_downtime=scheduled_downtime_list;this_downtime!=NULL;this_downtime=this_downtime->next){
+			if(this_downtime->triggered_by==temp_downtime->downtime_id)
+				handle_scheduled_downtime(this_downtime);
+		        }
 	        }
 	
 #ifdef DEBUG0
@@ -506,7 +465,7 @@ int handle_scheduled_downtime(scheduled_downtime *temp_downtime){
 
 
 /* checks for flexible (non-fixed) host downtime that should start now */
-int check_pending_flex_host_downtime(host *hst,int state){
+int check_pending_flex_host_downtime(host *hst){
 	scheduled_downtime *temp_downtime;
 	time_t current_time;
 
@@ -520,7 +479,7 @@ int check_pending_flex_host_downtime(host *hst,int state){
 	time(&current_time);
 
 	/* if host is currently up, nothing to do */
-	if(state==HOST_UP)
+	if(hst->current_state==HOST_UP)
 		return OK;
 
 	/* check all downtime entries */
@@ -535,8 +494,12 @@ int check_pending_flex_host_downtime(host *hst,int state){
 		if(temp_downtime->is_in_effect==TRUE)
 			continue;
 
+		/* triggered downtime entries should be ignored here */
+		if(temp_downtime->triggered_by!=0)
+			continue;
+
 		/* this entry matches our host! */
-		if(find_host(temp_downtime->host_name,NULL)==hst){
+		if(find_host(temp_downtime->host_name)==hst){
 			
 			/* if the time boundaries are okay, start this scheduled downtime */
 			if(temp_downtime->start_time<=current_time && current_time<=temp_downtime->end_time){
@@ -584,8 +547,12 @@ int check_pending_flex_service_downtime(service *svc){
 		if(temp_downtime->is_in_effect==TRUE)
 			continue;
 
+		/* triggered downtime entries should be ignored here */
+		if(temp_downtime->triggered_by!=0)
+			continue;
+
 		/* this entry matches our service! */
-		if(find_service(temp_downtime->host_name,temp_downtime->service_description,NULL)==svc){
+		if(find_service(temp_downtime->host_name,temp_downtime->service_description)==svc){
 
 			/* if the time boundaries are okay, start this scheduled downtime */
 			if(temp_downtime->start_time<=current_time && current_time<=temp_downtime->end_time){
@@ -606,7 +573,6 @@ int check_pending_flex_service_downtime(service *svc){
 /* checks for (and removes) expired downtime entries */
 int check_for_expired_downtime(void){
 	scheduled_downtime *temp_downtime;
-	scheduled_downtime *last_downtime;
 	scheduled_downtime *next_downtime;
 	time_t current_time;
 
@@ -617,35 +583,19 @@ int check_for_expired_downtime(void){
 	time(&current_time);
 
 	/* check all downtime entries... */
-	for(temp_downtime=scheduled_downtime_list,last_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=next_downtime){
+	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=next_downtime){
 
 		next_downtime=temp_downtime->next;
 
 		/* this entry should be removed */
 		if(temp_downtime->is_in_effect==FALSE && temp_downtime->end_time<current_time){
 
-			/* remove entry from the log */
-			delete_downtime(temp_downtime->type,temp_downtime->downtime_id);
-
-			/* delete the comment we added */
-			delete_comment((temp_downtime->type==HOST_DOWNTIME)?HOST_COMMENT:SERVICE_COMMENT,temp_downtime->comment_id);
-
-			/* remove the entry from the list in memory */
-			if(scheduled_downtime_list==temp_downtime)
-				scheduled_downtime_list=next_downtime;
+			/* delete the downtime entry */
+			if(temp_downtime->type==HOST_DOWNTIME)
+				delete_host_downtime(temp_downtime->downtime_id);
 			else
-				last_downtime->next=next_downtime;
-
-			/* free memory allocated to the entry */
-			free(temp_downtime->host_name);
-			free(temp_downtime->service_description);
-			free(temp_downtime->comment);
-			free(temp_downtime->author);
-			free(temp_downtime);
+				delete_service_downtime(temp_downtime->downtime_id);
 		        }
-
-		else
-			last_downtime=temp_downtime;
 	        }
 
 #ifdef DEBUG0
@@ -663,28 +613,38 @@ int check_for_expired_downtime(void){
 
 
 /* save a host or service downtime */
-int save_downtime(int type, char *host_name, char *service_description, time_t entry_time, char *author, char *comment, time_t start_time, time_t end_time, int fixed, unsigned long duration, int *downtime_id){
+int add_new_downtime(int type, char *host_name, char *service_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long *downtime_id){
 	int result;
 
 	if(type==HOST_DOWNTIME)
-		result=save_host_downtime(host_name,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+		result=add_new_host_downtime(host_name,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
 	else
-		result=save_service_downtime(host_name,service_description,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+		result=add_new_service_downtime(host_name,service_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
 
 	return result;
         }
 
 
 /* saves a host downtime entry */
-int save_host_downtime(char *host_name, time_t entry_time, char *author, char *comment, time_t start_time, time_t end_time, int fixed, unsigned long duration, int *downtime_id){
+int add_new_host_downtime(char *host_name, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long *downtime_id){
 	int result;
+	unsigned long new_downtime_id;
+
+	if(host_name==NULL)
+		return ERROR;
 
 	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
 #ifdef USE_XDDDEFAULT
-	result=xdddefault_save_host_downtime(host_name,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+	result=xdddefault_add_new_host_downtime(host_name,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,&new_downtime_id);
 #endif
-#ifdef USE_XDDDB
-	result=xdddb_save_host_downtime(host_name,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+
+	/* save downtime id */
+	if(downtime_id!=NULL)
+		*downtime_id=new_downtime_id;
+
+#ifdef USE_EVENT_BROKER
+	/* send data to event broker */
+	broker_downtime_data(NEBTYPE_DOWNTIME_ADD,NEBFLAG_NONE,NEBATTR_NONE,HOST_DOWNTIME,host_name,NULL,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,new_downtime_id,NULL);
 #endif
 
 	return result;
@@ -692,15 +652,25 @@ int save_host_downtime(char *host_name, time_t entry_time, char *author, char *c
 
 
 /* saves a service downtime entry */
-int save_service_downtime(char *host_name, char *service_description, time_t entry_time, char *author, char *comment, time_t start_time, time_t end_time, int fixed, unsigned long duration, int *downtime_id){
+int add_new_service_downtime(char *host_name, char *service_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long *downtime_id){
 	int result;
+	unsigned long new_downtime_id;
+
+	if(host_name==NULL || service_description==NULL)
+		return ERROR;
 
 	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
 #ifdef USE_XDDDEFAULT
-	result=xdddefault_save_service_downtime(host_name,service_description,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+	result=xdddefault_add_new_service_downtime(host_name,service_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,&new_downtime_id);
 #endif
-#ifdef USE_XDDDB
-	result=xdddb_save_service_downtime(host_name,service_description,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+
+	/* save downtime id */
+	if(downtime_id!=NULL)
+		*downtime_id=new_downtime_id;
+
+#ifdef USE_EVENT_BROKER
+	/* send data to event broker */
+	broker_downtime_data(NEBTYPE_DOWNTIME_ADD,NEBFLAG_NONE,NEBATTR_NONE,SERVICE_DOWNTIME,host_name,service_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,new_downtime_id,NULL);
 #endif
 
 	return result;
@@ -713,46 +683,86 @@ int save_service_downtime(char *host_name, char *service_description, time_t ent
 /******************************************************************/
 
 
-/* deletes a scheduled host or service downtime entry */
-int delete_downtime(int type,int downtime_id){
+/* deletes a scheduled host or service downtime entry from the list in memory */
+int delete_downtime(int type,unsigned long downtime_id){
 	int result;
+	scheduled_downtime *this_downtime=NULL;
+	scheduled_downtime *last_downtime=NULL;
+	scheduled_downtime *next_downtime=NULL;
 
-	if(type==HOST_DOWNTIME)
-		result=delete_host_downtime(downtime_id);
+	/* find the downtime we should remove */
+	for(this_downtime=scheduled_downtime_list,last_downtime=scheduled_downtime_list;this_downtime!=NULL;this_downtime=next_downtime){
+		next_downtime=this_downtime->next;
+
+		/* we found the downtime we should delete */
+		if(this_downtime->downtime_id==downtime_id && this_downtime->type==type)
+			break;
+
+		last_downtime=this_downtime;
+	        }
+
+	/* remove the downtime from the list in memory */
+	if(this_downtime!=NULL){
+
+		/* first remove the comment associated with this downtime */
+		if(this_downtime->type==HOST_DOWNTIME)
+			delete_host_comment(this_downtime->comment_id);
+		else
+			delete_service_comment(this_downtime->comment_id);
+
+#ifdef USE_EVENT_BROKER
+		/* send data to event broker */
+		broker_downtime_data(NEBTYPE_DOWNTIME_DELETE,NEBFLAG_NONE,NEBATTR_NONE,type,this_downtime->host_name,this_downtime->service_description,this_downtime->entry_time,this_downtime->author,this_downtime->comment,this_downtime->start_time,this_downtime->end_time,this_downtime->fixed,this_downtime->triggered_by,this_downtime->duration,downtime_id,NULL);
+#endif
+
+		if(scheduled_downtime_list==this_downtime)
+			scheduled_downtime_list=this_downtime->next;
+		else
+			last_downtime->next=next_downtime;
+		
+		/* free memory */
+		free(this_downtime->host_name);
+		free(this_downtime->service_description);
+		free(this_downtime->author);
+		free(this_downtime->comment);
+		free(this_downtime);
+
+		result=OK;
+	        }
 	else
-		result=delete_service_downtime(downtime_id);
-
+		result=ERROR;
+	
 	return result;
         }
 
 
 /* deletes a scheduled host downtime entry */
-int delete_host_downtime(int downtime_id){
+int delete_host_downtime(unsigned long downtime_id){
 	int result;
 
+	/* delete the downtime from memory */
+	delete_downtime(HOST_DOWNTIME,downtime_id);
+	
 	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
 #ifdef USE_XDDDEFAULT
 	result=xdddefault_delete_host_downtime(downtime_id);
 #endif
-#ifdef USE_XDDDB
-	result=xdddb_delete_host_downtime(downtime_id,FALSE);
-#endif
 
 	return result;
         }
 
 
 /* deletes a scheduled service downtime entry */
-int delete_service_downtime(int downtime_id){
+int delete_service_downtime(unsigned long downtime_id){
 	int result;
 
+	/* delete the downtime from memory */
+	delete_downtime(SERVICE_DOWNTIME,downtime_id);
+	
 	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
 #ifdef USE_XDDDEFAULT
 	result=xdddefault_delete_service_downtime(downtime_id);
 #endif
-#ifdef USE_XDDDB
-	result=xdddb_delete_service_downtime(downtime_id,FALSE);
-#endif
 
 	return result;
         }
@@ -775,9 +785,6 @@ int read_downtime_data(char *main_config_file){
 #ifdef USE_XDDDEFAULT
 	result=xdddefault_read_downtime_data(main_config_file);
 #endif
-#ifdef USE_XDDDB
-	result=xdddb_read_downtime_data(main_config_file);
-#endif
 
 	return result;
         }
@@ -791,31 +798,35 @@ int read_downtime_data(char *main_config_file){
 /******************************************************************/
 
 /* adds a host downtime entry to the list in memory */
-int add_host_downtime(char *host_name, time_t entry_time, char *author, char *comment, time_t start_time, time_t end_time, int fixed, unsigned long duration, int downtime_id){
+int add_host_downtime(char *host_name, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long downtime_id){
 	int result;
 
-	result=add_downtime(HOST_DOWNTIME,host_name,NULL,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+	result=add_downtime(HOST_DOWNTIME,host_name,NULL,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
 
 	return result;
         }
 
 
 /* adds a service downtime entry to the list in memory */
-int add_service_downtime(char *host_name, char *svc_description, time_t entry_time, char *author, char *comment, time_t start_time, time_t end_time, int fixed, unsigned long duration, int downtime_id){
+int add_service_downtime(char *host_name, char *svc_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long downtime_id){
 	int result;
 
-	result=add_downtime(SERVICE_DOWNTIME,host_name,svc_description,entry_time,author,comment,start_time,end_time,fixed,duration,downtime_id);
+	result=add_downtime(SERVICE_DOWNTIME,host_name,svc_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
 
 	return result;
         }
 
 
 /* adds a host or service downtime entry to the list in memory */
-int add_downtime(int downtime_type, char *host_name, char *svc_description, time_t entry_time, char *author, char *comment, time_t start_time, time_t end_time, int fixed, unsigned long duration, int downtime_id){
+int add_downtime(int downtime_type, char *host_name, char *svc_description, time_t entry_time, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long triggered_by, unsigned long duration, unsigned long downtime_id){
 	scheduled_downtime *new_downtime=NULL;
 	scheduled_downtime *last_downtime=NULL;
 	scheduled_downtime *temp_downtime=NULL;
 
+	/* don't add triggered downtimes that don't have a valid parent */
+	if(triggered_by>0  && find_downtime(ANY_DOWNTIME,triggered_by)==NULL)
+		return ERROR;
+
 	/* allocate memory for the downtime */
 	new_downtime=(scheduled_downtime *)malloc(sizeof(scheduled_downtime));
 	if(new_downtime==NULL)
@@ -843,16 +854,17 @@ int add_downtime(int downtime_type, char *host_name, char *svc_description, time
 	else
 		new_downtime->author=strdup(author);
 
-	if(comment==NULL)
+	if(comment_data==NULL)
 		new_downtime->comment=NULL;
 	else
-		new_downtime->comment=strdup(comment);
+		new_downtime->comment=strdup(comment_data);
 
 	new_downtime->type=downtime_type;
 	new_downtime->entry_time=entry_time;
 	new_downtime->start_time=start_time;
 	new_downtime->end_time=end_time;
 	new_downtime->fixed=(fixed>0)?TRUE:FALSE;
+	new_downtime->triggered_by=triggered_by;
 	new_downtime->duration=duration;
 	new_downtime->downtime_id=downtime_id;
 #ifdef NSCORE
@@ -886,6 +898,13 @@ int add_downtime(int downtime_type, char *host_name, char *svc_description, time
 		last_downtime->next=new_downtime;
 	        }
 
+#ifdef NSCORE
+#ifdef USE_EVENT_BROKER
+	/* send data to event broker */
+	broker_downtime_data(NEBTYPE_DOWNTIME_LOAD,NEBFLAG_NONE,NEBATTR_NONE,downtime_type,host_name,svc_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id,NULL);
+#endif
+#endif
+
 	return OK;
         }
 
@@ -896,12 +915,12 @@ int add_downtime(int downtime_type, char *host_name, char *svc_description, time
 /************************ SEARCH FUNCTIONS ************************/
 /******************************************************************/
 
-/* finds a specific host downtime entry */
-scheduled_downtime *find_host_downtime(int downtime_id){
+/* finds a specific downtime entry */
+scheduled_downtime *find_downtime(int type, unsigned long downtime_id){
 	scheduled_downtime *temp_downtime;
 
 	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
-		if(temp_downtime->type!=HOST_DOWNTIME)
+		if(type!=ANY_DOWNTIME && temp_downtime->type!=type)
 			continue;
 		if(temp_downtime->downtime_id==downtime_id)
 			return temp_downtime;
@@ -911,18 +930,17 @@ scheduled_downtime *find_host_downtime(int downtime_id){
         }
 
 
+/* finds a specific host downtime entry */
+scheduled_downtime *find_host_downtime(unsigned long downtime_id){
+
+	return find_downtime(HOST_DOWNTIME,downtime_id);
+        }
+
+
 /* finds a specific service downtime entry */
-scheduled_downtime *find_service_downtime(int downtime_id){
-	scheduled_downtime *temp_downtime;
+scheduled_downtime *find_service_downtime(unsigned long downtime_id){
 
-	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
-		if(temp_downtime->type!=SERVICE_DOWNTIME)
-			continue;
-		if(temp_downtime->downtime_id==downtime_id)
-			return temp_downtime;
-	        }
-
-	return NULL;
+	return find_downtime(SERVICE_DOWNTIME,downtime_id);
         }
 
 
@@ -941,6 +959,8 @@ void free_downtime_data(void){
 		next_downtime=this_downtime->next;
 		free(this_downtime->host_name);
 		free(this_downtime->service_description);
+		free(this_downtime->author);
+		free(this_downtime->comment);
 		free(this_downtime);
 	        }
 
